@@ -25,19 +25,16 @@
 #' @return a simple features (sf) object
 #' @keywords internal
 #' @importFrom sf st_crs st_geometry_type st_buffer st_transform st_zm read_sf st_bbox st_as_sfc
+#' @importFrom sf sf_use_s2
 #' @importFrom httr POST RETRY
 #' @importFrom dplyr filter
 #' @importFrom methods as
+#' @importFrom xml2 read_xml
 
-query_usgs_geoserver <- function(AOI = NULL,  ids = NULL,
+query_usgs_geoserver <- memoise::memoise(function(AOI = NULL,  ids = NULL,
                                  type = NULL, filter = NULL,
                                  t_srs = NULL,
-                                 buffer = 0.5){
-
-  # If t_src is not provided set to AOI CRS
-  if(is.null(t_srs)){ t_srs  <- sf::st_crs(AOI)}
-  # If AOI CRS is NA (e.g st_crs(NULL)) then set to 4326
-  if(is.na(t_srs))  { t_srs  <- 4326}
+                                 buffer = 0.5) {
 
   source <- data.frame(server = 'wmadata',
                        user_call  = c('huc02', 'huc04', 'huc06',
@@ -80,44 +77,22 @@ query_usgs_geoserver <- function(AOI = NULL,  ids = NULL,
 
   if(is.null(type)){ return(source) }
 
-  if(!is.null(AOI) & !is.null(ids)){
-    # Check if AOI and IDs are both given
-    stop("Either IDs or a spatial AOI can be passed.", .call = FALSE)
-  } else if(is.null(AOI) & is.null(ids)){
-    # Check if AOI and IDs are both NULL
-    stop("IDs or a spatial AOI must be passed.",.call = FALSE)
-  } else if(!(type %in% source$user_call)) {
-    # Check that "type" is valid
-    stop(paste("Type not available must be one of:",
-               paste(source$user_call, collapse = ", ")),
-         call. = FALSE)
-  }
+  AOI <- check_query_params(AOI, ids, type, NULL, source, t_srs, buffer)
+  t_srs <- AOI$t_srs
+  AOI <- AOI$AOI
 
-  if(!is.null(AOI)){
+  here <- filter(source, .data$user_call == !!type)
 
-    if(length(sf::st_geometry(AOI)) > 1) {
-      stop("AOI must be one an only one feature.")
-    }
+  URL <- paste0(get("geoserver_root", envir = nhdplusTools_env),
+                here$server,
+                "/ows")
 
-    if(sf::st_geometry_type(AOI) == "POINT"){
-      # If input is a POINT, buffer by 1/2 meter (in equal area projection)
-      AOI = sf::st_buffer(sf::st_transform(AOI, 5070), buffer) %>%
-        sf::st_bbox() %>% sf::st_as_sfc() %>% sf::st_make_valid()
-    }
-  }
+  use_s2 <- sf_use_s2()
+  sf_use_s2(FALSE)
 
-  here     <- dplyr::filter(source, .data$user_call == !!type)
+  on.exit(sf_use_s2(use_s2), add = TRUE)
 
-  URL      <- paste0(get("geoserver_root", envir = nhdplusTools_env),
-                     here$server,
-                     "/ows")
-
-  use_s2 <- sf::sf_use_s2()
-  sf::sf_use_s2(FALSE)
-
-  on.exit(sf::sf_use_s2(use_s2), add = TRUE)
-
-  out <- spatial_filter(AOI, type = here$geoserver, tile = here$page, geom_name = here$geom_name)
+  out <- spatial_filter(AOI, tile = here$page, geom_name = here$geom_name)
 
   for(i in 1:length(out)) {
 
@@ -156,19 +131,19 @@ query_usgs_geoserver <- function(AOI = NULL,  ids = NULL,
     tryCatch({
       if(nhdplus_debug()) {
         message(paste(URL, "\n"))
-        message(as.character(xml2::read_xml(filterXML)))
+        message(as.character(read_xml(filterXML)))
       }
 
-      out[[i]] <- rawToChar(httr::RETRY("POST",
-                                    URL,
-                                    body = filterXML)$content)
+      out[[i]] <- rawToChar(RETRY("POST",
+                                  URL,
+                                  body = filterXML)$content)
     }, error = function(e) {
       warning("Something went wrong trying to access a service.")
       return(NULL)
     })
 
     out[[i]] <- tryCatch({
-      sf::st_zm(sf::read_sf(out[[i]]))},
+      st_zm(read_sf(out[[i]]))},
       error = function(e) return(NULL))
   }
 
@@ -188,8 +163,8 @@ query_usgs_geoserver <- function(AOI = NULL,  ids = NULL,
 
   } else if(!is.null(AOI)){
 
-    out = sf::st_filter(sf::st_transform(out, t_srs),
-                        sf::st_transform(AOI, t_srs))
+    out = st_filter(st_transform(out, t_srs),
+                    st_transform(AOI, t_srs))
     if(nrow(out) == 0){
       out = NULL
     }
@@ -202,7 +177,7 @@ query_usgs_geoserver <- function(AOI = NULL,  ids = NULL,
     NULL
   }
 
-}
+}, ~memoise::timeout(nhdplusTools_memoise_timeout()), cache = nhdplusTools_memoise_cache())
 
 unify_types <- function(out) {
   all_class <- bind_rows(lapply(out, function(x) {
@@ -227,6 +202,12 @@ unify_types <- function(out) {
       }
     }
   }
+
+  rows <- sapply(out, nrow)
+
+  if(any(rows > 0))
+    out <- out[rows > 0]
+
   out
 }
 
@@ -244,36 +225,16 @@ assign("bb_break_size", value = 2, nhdplusTools_env)
 #' @noRd
 #' @importFrom sf st_geometry_type st_buffer st_transform st_bbox
 
-spatial_filter  <- function(AOI, type = 'catchmentsp',
+spatial_filter  <- function(AOI,
                             break_size = get("bb_break_size", nhdplusTools_env),
                             tile = TRUE,
-                            geom_name = "the_geom"){
+                            geom_name = "the_geom", format = 'ogc') {
 
   if(is.null(AOI)) return(list(list()))
 
-  # "The current GeoServer implementation will return all the streets whose
-  # ENVELOPE overlaps the BBOX provided. This may includes some nearby
-  # streets that do not strictly fall within the bounding box.
-  # This is a fast response - if a more accurate but slower response is required,
-  # use Intersects or Within."
 
-  # if (as.character(sf::st_geometry_type(AOI)) == "POINT") {
-  #
-  #   AOI <- sf::st_coordinates(AOI)
-  #
-  #   return(paste0("?service=WFS",
-  #                 "&version=1.0.0",
-  #                 "&request=GetFeature",
-  #                 "&typeName=wmadata:", type,
-  #                 "&outputFormat=application%2Fjson",
-  #                 "&srsName=EPSG:4326",
-  #                 "&CQL_FILTER=INTERSECTS%28the_geom,%20POINT%20%28",
-  #                 AOI[1], "%20", AOI[2], "%29%29"))
-  #   }
-
-
-  bb <- sf::st_transform(AOI, 4326) %>%
-    sf::st_bbox()
+  bb <- st_transform(AOI, 4326) %>%
+    st_bbox()
 
   if(tile) {
     x_breaks <- seq(bb$xmin, bb$xmax, length.out = (ceiling((bb$xmax - bb$xmin) / break_size) + 1))
@@ -287,15 +248,37 @@ spatial_filter  <- function(AOI, type = 'catchmentsp',
   } else {
     bb_list <- list(bb)
   }
-  lapply(bb_list, function(bb) {
-    paste0('<ogc:BBOX>',
-           '<ogc:PropertyName>', geom_name, '</ogc:PropertyName>',
-           '<gml:Envelope srsName="urn:x-ogc:def:crs:EPSG:4326">',
-           '<gml:lowerCorner>', bb$ymin, " ", bb$xmin, '</gml:lowerCorner>',
-           '<gml:upperCorner>', bb$ymax, " ", bb$xmax, '</gml:upperCorner>',
-           '</gml:Envelope>',
-           '</ogc:BBOX>')
-  })
+
+  if(format == "ogc") {
+    lapply(bb_list, function(bb) {
+      paste0('<ogc:BBOX>',
+             '<ogc:PropertyName>', geom_name, '</ogc:PropertyName>',
+             '<gml:Envelope srsName="urn:x-ogc:def:crs:EPSG:4326">',
+             '<gml:lowerCorner>', bb$ymin, " ", bb$xmin, '</gml:lowerCorner>',
+             '<gml:upperCorner>', bb$ymax, " ", bb$xmax, '</gml:upperCorner>',
+             '</gml:Envelope>',
+             '</ogc:BBOX>')
+    })
+  } else if(format == "esri") {
+
+    # {
+    #   "xmin": -109.55,
+    #   "ymin": 25.76,
+    #   "xmax": -86.39,
+    #   "ymax": 49.94,
+    #   "spatialReference": {
+    #     "wkid": 4326
+    #   }
+    # }
+
+    lapply(bb_list, function(bb) {
+      jsonlite::toJSON(
+        c(bb, list(spatialReference = list(wkid = 4326))),
+        auto_unbox = TRUE)
+    })
+  } else {
+    stop("format must be ogc or esri")
+  }
 }
 
 #' @title Construct an attribute filter for geoservers
@@ -376,7 +359,7 @@ tc <- function(x) {
 #' @importFrom httr RETRY GET
 #' @importFrom jsonlite fromJSON
 
-extact_comid_nwis <- function(nwis){
+extact_comid_nwis <- memoise::memoise(function(nwis){
   # We could export this from dataRetrieval dataRetrieval:::pkg.env$nldi_base
   #but currently its not...
   baseURL  <- paste0(get_nldi_url(), "/linked-data/")
@@ -384,4 +367,40 @@ extact_comid_nwis <- function(nwis){
   c        <-  rawToChar(httr::RETRY("GET", url)$content)
   f.comid  <-  jsonlite::fromJSON(c, simplifyVector = TRUE)
   f.comid$features$properties$comid
+}, ~memoise::timeout(nhdplusTools_memoise_timeout()), cache = nhdplusTools_memoise_cache())
+
+#' @importFrom sf st_make_valid st_as_sfc st_bbox st_buffer st_transform
+check_query_params <- function(AOI, ids, type, where, source, t_srs, buffer) {
+  # If t_src is not provided set to AOI CRS
+  if(is.null(t_srs)){ t_srs  <- st_crs(AOI) }
+  # If AOI CRS is NA (e.g st_crs(NULL)) then set to 4326
+  if(is.na(t_srs))  { t_srs  <- 4326 }
+
+  if(!is.null(AOI) & !is.null(ids)) {
+    # Check if AOI and IDs are both given
+    stop("Either IDs or a spatial AOI can be passed.", .call = FALSE)
+  } else if(is.null(AOI) & is.null(ids) & !(!is.null(where) && grepl("IN", where))) {
+    # Check if AOI and IDs are both NULL
+    stop("IDs or a spatial AOI must be passed.", .call = FALSE)
+  } else if(!(type %in% source$user_call)) {
+    # Check that "type" is valid
+    stop(paste("Type not available must be one of:",
+               paste(source$user_call, collapse = ", ")),
+         call. = FALSE)
+  }
+
+  if(!is.null(AOI)){
+
+    if(length(st_geometry(AOI)) > 1) {
+      stop("AOI must be one an only one feature.")
+    }
+
+    if(st_geometry_type(AOI) == "POINT"){
+      # If input is a POINT, buffer by 1/2 meter (in equal area projection)
+      AOI = st_buffer(st_transform(AOI, 5070), buffer) %>%
+        st_bbox() %>% st_as_sfc() %>% st_make_valid()
+    }
+  }
+
+  return(list(AOI = AOI, t_srs = t_srs))
 }
